package org.qtunes.auth;

import org.qtunes.core.*;
import java.util.*;
import java.io.*;

/**
 * A simple Authorizer that tests a Session aganst a {@link SessionMatcher}.
 * If the session is found to match, the allowances specified in the
 * constructor are granted.
 */
public class BasicAuthorizer implements Authorizer {

    private final Set<String> allowances;
    private final SessionMatcher matcher;

    public BasicAuthorizer(SessionMatcher matcher, String[] allowances) {
        this.allowances = new HashSet<String>();
        this.allowances.addAll(Arrays.asList(allowances));
        this.matcher = matcher;
    }

    public int hashCode() {
        return matcher.hashCode();
    }

    public boolean equals(Object o) {
        return o instanceof BasicAuthorizer ? ((BasicAuthorizer)o).matcher.equals(matcher) : false;
    }

    public Collection<String> getAllowances(Session session) {
        if (matcher.matches(session)) {
            return allowances;
        } else {
            return null;
        }
    }

    public String toString() {
        return "{auth:"+matcher+", allows:"+allowances+"}";
    }

}
