package org.qtunes.auth.control;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.db.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import org.apache.log4j.Logger;

/**
 * A ParamParser suitable for interactive use.
 * Requests are made as you would invoke a method in JavaScript, eg "Player.addTrack(3)"
 * or "db.getField(1, 'Title')". Responses are formatted as plain text.
 */
public abstract class SimpleParser implements ParamParser {

    private ControlJSONSerializer json = new ControlJSONSerializer();
    private String object;
    private List<Call> calls;

    @SuppressWarnings("unchecked")
    public void parse(String in) throws Exception {
        object = null;
        calls = new ArrayList<Call>();
        int i = in.indexOf(".");
        if (i < 0) {
            i = in.length();
        }
        object = in.substring(0, i);
        char quote = 0;
        int depth = 0, start = i;
        String methodname = null;
        for (;i<in.length();i++) {
            char c = in.charAt(i);
            if (c=='\\') {
                i++;
            } else if ((c=='\'' || c=='"') && quote==0) {
                quote = c;
            } else if (c==quote) {
                quote = 0;
            } else if (quote==0 && c=='(' && depth++==0) {
                methodname = in.substring(start, i);
            } else if (quote==0 && c==')' && --depth==0) {
                String argstring = in.substring(start+methodname.length()+1, i);
                Map args;
                if (argstring.length()==0) {
                    args = Collections.EMPTY_MAP;
                } else if (argstring.charAt(0)=='{') {
                    args = (Map)json.read(argstring);
                } else {
                    List list = (List)json.read("["+argstring+"]");
                    args = new LinkedHashMap();
                    for (int j=0;j<list.size();j++) {
                        args.put(Integer.valueOf(j), list.get(j));
                    }
                }
                if (methodname.charAt(0)!='.') {
                    throw new IllegalArgumentException("Invalid input \""+in+"\" (1)");
                }
                calls.add(new Call(methodname.substring(1), args));
                methodname = null;
                start = i+1;
            }
        }
        if (start!=in.length()) {
            throw new IllegalArgumentException("Invalid input \""+in+"\" (2)");
        }
    }

    public String getObjectName() {
        return object;
    }

    public List<Call> getMethodCalls() {
        return calls;
    }

    public abstract Session getSession();

    public String toString(Object o) {
        if (o==Void.TYPE) {
            return null;
        } else if (o instanceof Throwable) {
            StringWriter sw = new StringWriter();
            ((Throwable)o).printStackTrace(new PrintWriter(sw));
            return sw.toString();
        } else {
            return json.write(o);
        }
    }

}
