package org.qtunes.core;

import java.lang.annotation.*;

/**
 * The ControlMethod annotation marks a method as being
 * publically callable via one of the Controllers (which
 * will call the {@link org.qtunes.control.ControlMethodInvoker}
 * to do it's work.
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface ControlMethod { 
    /**
     * A list of parameter names in the order the method takes them
     */
    String[] params();

    /**
     * The authorization required to run this method. The default is
     * "" which means no session is required. A value of "*" means
     * any valid session will do, and other values like "play" or "edit"
     * will require a Session with that allowance.
     */
    String auth() default "";

    /**
     * A description of the method
     */
    String help();
}
