package org.qtunes.core.util;

public class Base64 {

    private static final String chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

    private Base64() {}

    public static String encode(byte[] in) {
        return encode(in, true);
    }

    /**
     * Encode the bytes as a Base64 String
     * @param addequals whether to add the "=" at the end or not
     */
    public static String encode(byte[] in, boolean addequals) {
        final int len = in.length;
        final int olen = (len*4+2)/3;
        StringBuffer sb = new StringBuffer(((len+2)/3)*4);
        for (int i=0;i<len;) {
            int i0 = in[i++] & 0xff;
            int i1 = i < len ? in[i++] & 0xff : 0;
            int i2 = i < len ? in[i++] & 0xff : 0;
            int o0 = i0 >>> 2;
            int o1 = ((i0 & 3)  << 4) | (i1 >>> 4);
            int o2 = ((i1 & 0xf) << 2) | (i2 >>> 6);
            int o3 = i2 & 0x3f;
            sb.append(chars.charAt(o0));
            sb.append(chars.charAt(o1));
            sb.append(sb.length() < olen ? chars.charAt(o2) : addequals ? '=' : ' ');
            sb.append(sb.length() < olen ? chars.charAt(o3) : addequals ? '=' : ' ');
        }
        return sb.toString().trim();
    }

    /**
     * Decode the Base64 String to bytes
     */
    public static byte[] decode(CharSequence in) {
        int len = 0;
        for (int i=0;i < in.length();i++) {
            if (chars.indexOf(in.charAt(i)) >= 0) {
                len++;
            }
        }
        byte[] out = new byte[len / 4 * 3 + Math.max(0, len%4 - 1)];
        int s=0, a=0, j=0;
        for (int i=0;i<in.length();i++) {
            int v = chars.indexOf(in.charAt(i));
            if (v >= 0) {
                a  = (a<<6) | v;
                s += 6;
                if (s >= 8) {
                    s -= 8;
                    out[j++] = (byte)((a >> s) & 0xFF);
                }
            }
        }
        return out;
    }

    /**
     * Return a printable version of the specified String
     */
    public static final String toPrintableString(CharSequence in) {
        StringBuffer sb = new StringBuffer();
        for (int i=0;i<in.length();i++) {
            char c = in.charAt(i);
            if (c=='\\') {
                sb.append("\\\\");
            } else if (c>=32 && c<=127) {
                sb.append(c);
            } else if (c=='\n') {
                sb.append("\\n");
            } else if (c=='\r') {
                sb.append("\\r");
            } else if (c=='\t') {
                sb.append("\\t");
            } else if (c==0) {
                sb.append("\\0");
            } else if (c > 0xFFF) {
                sb.append("\\u"+Integer.toHexString(c));
            } else if (c > 0xFF) {
                sb.append("\\u0"+Integer.toHexString(c));
            } else if (c > 0xF) {
                sb.append("\\u00"+Integer.toHexString(c));
            } else {
                sb.append("\\u000"+Integer.toHexString(c));
            }
        }
        return sb.toString();
    }

    /**
     * Return a hex String of the specified byte array
     */
    public static final String toPrintableString(byte[] in) {
        StringBuffer sb = new StringBuffer(in.length * 2);
        for (int i=0;i<in.length;i++) {
            String s = Integer.toHexString(in[i]&0xFF);
            if (s.length()==1) sb.append('0');
            sb.append(s);
        }
        return sb.toString();
    }

}
