package org.qtunes.daap;

import org.qtunes.db.*;
import java.io.*;
import java.util.*;


/**
 * Utility class containing all the Block definitions, and methods relating
 * to their use. The BlockDef inner class is a Block definition, with Number,
 * String and List subtypes. This "strong typing" is possibly overkill but
 * made it easy to catch adding a block of the wrong type.
 */
class Blocks {

    private static Map<Object,BlockDef> defmap = new HashMap<Object,BlockDef>();  // Keys are Strings and Integers
    private static List<BlockDef> contentcodes = new ArrayList<BlockDef>();

    static final int BYT=1;
    static final int BYS=2;
    static final int SHO=3;
    static final int SHS=4;
    static final int INT=5;
    static final int INS=6;
    static final int LON=7;
    static final int STR=9;
    static final int DAT=10;
    static final int VER=11;
    static final int LIS=12;

    // Define the blocks.
    public static final BlockDefL abal = new BlockDefL("abal", LIS, "daap.browsealbumlisting");
    public static final BlockDefL abar = new BlockDefL("abar", LIS, "daap.browseartistlisting");
    public static final BlockDefL abcp = new BlockDefL("abcp", LIS, "daap.browsecomposerlisting");
    public static final BlockDefL abgn = new BlockDefL("abgn", LIS, "daap.browsegenrelisting");
    public static final BlockDefN abpl = new BlockDefN("abpl", BYT, "daap.baseplaylist");
    public static final BlockDefL abro = new BlockDefL("abro", LIS, "daap.databasebrowse");
    public static final BlockDefL adbs = new BlockDefL("adbs", LIS, "daap.databasesongs");
    public static final BlockDefN aeAI = new BlockDefN("aeAI", INT, "com.apple.itunes.itms-artistid");
    public static final BlockDefN aeCI = new BlockDefN("aeCI", INT, "com.apple.itunes.itms-composerid");
    public static final BlockDefS aeCR = new BlockDefS("aeCR", STR, "com.apple.itunes.content-rating");
    public static final BlockDefN aeDP = new BlockDefN("aeDP", INT, "com.apple.itunes.drm-platform-id");
    public static final BlockDefN aeDR = new BlockDefN("aeDR", LON, "com.apple.itunes.drm-user-id");
    public static final BlockDefN aeDV = new BlockDefN("aeDV", INT, "com.apple.itunes.drm-versions");
    public static final BlockDefS aeEN = new BlockDefS("aeEN", STR, "com.apple.itunes.episode-num-str");
    public static final BlockDefN aeES = new BlockDefN("aeES", INT, "com.apple.itunes.episode-sort");
    public static final BlockDefN aeFP = new BlockDefN("aeFP", BYT);                                    
    public static final BlockDefN aeGD = new BlockDefN("aeGD", INT, "com.apple.itunes.gapless-enc-dr");
    public static final BlockDefN aeGE = new BlockDefN("aeGE", INT, "com.apple.itunes.gapless-enc-del");
    public static final BlockDefN aeGH = new BlockDefN("aeGH", INT, "com.apple.itunes.gapless-heur");
    public static final BlockDefN aeGI = new BlockDefN("aeGI", INT, "com.apple.itunes.itms-genreid");
    public static final BlockDefN aeGR = new BlockDefN("aeGR", LON, "com.apple.itunes.gapless-resy");
    public static final BlockDefN aeGU = new BlockDefN("aeGU", LON, "com.apple.iTunes.gapless-dur");
    public static final BlockDefN aeHV = new BlockDefN("aeHV", BYT, "com.apple.itunes.has-video");
    public static final BlockDefN aeK1 = new BlockDefN("aeK1", LON, "com.apple.itunes.drm-key1-id");
    public static final BlockDefN aeK2 = new BlockDefN("aeK2", LON, "com.apple.itunes.drm-key2-id");
    public static final BlockDefN aeMK = new BlockDefN("aeMK", BYT, "com.apple.itunes.mediakind", DaapField.MediaKind);
    public static final BlockDefN aeMk = new BlockDefN("aeMk", INT, "com.apple.itunes.extended-media-kind");
    public static final BlockDefN aeND = new BlockDefN("aeND", LON, "com.apple.itunes.non-drm-user-id");
    public static final BlockDefS aeNN = new BlockDefS("aeNN", STR, "com.apple.itunes.network-name");
    public static final BlockDefN aeNV = new BlockDefN("aeNV", INT, "com.apple.itunes.norm-volume");
    public static final BlockDefN aePC = new BlockDefN("aePC", BYT, "com.apple.itunes.is-podcast");
    public static final BlockDefN aePI = new BlockDefN("aePI", INT, "com.apple.itunes.itms-playlistid");
    public static final BlockDefN aePP = new BlockDefN("aePP", BYT, "com.apple.itunes.is-podcast-playlist");
    public static final BlockDefN aePS = new BlockDefN("aePS", BYT, "com.apple.itunes.special-playlist");
    public static final BlockDefN aeSE = new BlockDefN("aeSE", LON, "com.apple.itunes.store-pers-id");
    public static final BlockDefN aeSF = new BlockDefN("aeSF", INT, "com.apple.itunes.itms-storefrontid");
    public static final BlockDefN aeSG = new BlockDefN("aeSG", BYT, "com.apple.itunes.saved-genius");
    public static final BlockDefN aeSI = new BlockDefN("aeSI", INT, "com.apple.itunes.itms-songid");
    public static final BlockDefS aeSN = new BlockDefS("aeSN", STR, "com.apple.itunes.series-name");
    public static final BlockDefN aeSP = new BlockDefN("aeSP", BYT, "com.apple.itunes.smart-playlist");
    public static final BlockDefN aeSU = new BlockDefN("aeSU", INT, "com.apple.itunes.season-num");
    public static final BlockDefN aeSV = new BlockDefN("aeSV", INT, "com.apple.itunes.music-sharing-version");
    public static final BlockDefS aeXD = new BlockDefS("aeXD", STR, "com.apple.itunes.xid");
    public static final BlockDefL agal = new BlockDefL("agal", LIS);
    public static final BlockDefS agrp = new BlockDefS("agrp", STR, "daap.songgrouping", Field.Group);
    public static final BlockDefL aply = new BlockDefL("aply", LIS, "daap.databaseplaylists");
    public static final BlockDefN aprm = new BlockDefN("aprm", BYT, "daap.playlistrepeatmode");
    public static final BlockDefN apro = new BlockDefN("apro", VER, "daap.protocolversion");
    public static final BlockDefN apsm = new BlockDefN("apsm", BYT, "daap.playlistshufflemode");
    public static final BlockDefL apso = new BlockDefL("apso", LIS, "daap.playlistsongs");
    public static final BlockDefL arif = new BlockDefL("arif", LIS, "daap.resolveinfo");
    public static final BlockDefL arsv = new BlockDefL("arsv", LIS, "daap.resolve");
    public static final BlockDefS asaa = new BlockDefS("asaa", STR, "daap.songalbumartist", Field.AlbumArtist);
    public static final BlockDefN asai = new BlockDefN("asai", LON, "daap.songalbumid", DaapField.AlbumId);
    public static final BlockDefS asal = new BlockDefS("asal", STR, "daap.songalbum", Field.Album);
    public static final BlockDefS asar = new BlockDefS("asar", STR, "daap.songartist", Field.Artist);
    public static final BlockDefN asbk = new BlockDefN("asbk", BYT, "daap.bookmarkable");
    public static final BlockDefN asbo = new BlockDefN("asbo", INT, "daap.songbookmark");
    public static final BlockDefN asbr = new BlockDefN("asbr", SHO, "daap.songbitrate", Field.BitRate);
    public static final BlockDefN asbt = new BlockDefN("asbt", SHO, "daap.songbeatsperminute", Field.BPM);
    public static final BlockDefN ascd = new BlockDefN("ascd", INT, "daap.songcodectype");
    public static final BlockDefS ascm = new BlockDefS("ascm", STR, "daap.songcomment", Field.Comment);
    public static final BlockDefS ascn = new BlockDefS("ascn", STR, "daap.songcontentdescription");
    public static final BlockDefN asco = new BlockDefN("asco", BYT, "daap.songcompilation", Field.Compilation);
    public static final BlockDefS ascp = new BlockDefS("ascp", STR, "daap.songcomposer", Field.Composer);
    public static final BlockDefN ascr = new BlockDefN("ascr", BYT, "daap.songcontentrating");
    public static final BlockDefN ascs = new BlockDefN("ascs", INT, "daap.songcodecsubtype");
    public static final BlockDefS asct = new BlockDefS("asct", STR, "daap.songcategory");
    public static final BlockDefN asda = new BlockDefN("asda", DAT, "daap.songdateadded", Field.Added);
    public static final BlockDefN asdb = new BlockDefN("asdb", BYT, "daap.songdisabled");
    public static final BlockDefN asdc = new BlockDefN("asdc", SHO, "daap.songdisccount", Field.NumDiscs);
    public static final BlockDefN asdk = new BlockDefN("asdk", BYT, "daap.songdatakind", DaapField.SongDataKind);
    public static final BlockDefN asdm = new BlockDefN("asdm", DAT, "daap.songdatemodified", Field.LastModified);
    public static final BlockDefN asdn = new BlockDefN("asdn", SHO, "daap.songdiscnumber", Field.Disc);
    public static final BlockDefN asdp = new BlockDefN("asdp", DAT, "daap.songdatepurchased");
    public static final BlockDefN asdr = new BlockDefN("asdr", DAT, "daap.songdatereleased");
    public static final BlockDefS asdt = new BlockDefS("asdt", STR, "daap.songdescription", DaapField.MediaDescription);
    public static final BlockDefN ased = new BlockDefN("ased", SHO, "daap.songextradata");
    public static final BlockDefS aseq = new BlockDefS("aseq", STR, "daap.songeqpreset");
    public static final BlockDefS asfm = new BlockDefS("asfm", STR, "daap.songformat", Field.Type);
    public static final BlockDefS asgn = new BlockDefS("asgn", STR, "daap.songgenre", Field.Genre);
    public static final BlockDefN asgp = new BlockDefN("asgp", BYT, "daap.songgapless", Field.Gapless);
    public static final BlockDefN ashp = new BlockDefN("ashp", BYT, "daap.songhasbeenplayed");
    public static final BlockDefS asky = new BlockDefS("asky", STR, "daap.songkeywords");
    public static final BlockDefS aslc = new BlockDefS("aslc", STR, "daap.songlongcontentdescription");
    public static final BlockDefN asls = new BlockDefN("asls", LON, "daap.songlongsize");
    public static final BlockDefS aspu = new BlockDefS("aspu", STR, "daap.songpodcasturl");
    public static final BlockDefN asrv = new BlockDefN("asrv", BYS, "daap.songrelativevolume");
    public static final BlockDefS assa = new BlockDefS("assa", STR, "daap.sortartist");
    public static final BlockDefS assc = new BlockDefS("assc", STR, "daap.sortcomposer");
    public static final BlockDefS assl = new BlockDefS("assl", STR, "daap.sortalbumartist");
    public static final BlockDefS assn = new BlockDefS("assn", STR, "daap.sortname");
    public static final BlockDefN assp = new BlockDefN("assp", INT, "daap.songstoptime");
    public static final BlockDefN assr = new BlockDefN("assr", INT, "daap.songsamplerate", Field.SampleRate);
    public static final BlockDefS asss = new BlockDefS("asss", STR, "daap.sortseriesname");
    public static final BlockDefN asst = new BlockDefN("asst", INT, "daap.songstarttime");
    public static final BlockDefS assu = new BlockDefS("assu", STR, "daap.sortalbum");
    public static final BlockDefN assz = new BlockDefN("assz", INT, "daap.songsize", Field.FileSize);
    public static final BlockDefN astc = new BlockDefN("astc", SHO, "daap.songtrackcount", Field.NumTracks);
    public static final BlockDefN astm = new BlockDefN("astm", INT, "daap.songtime", Field.Duration);
    public static final BlockDefN astn = new BlockDefN("astn", SHO, "daap.songtracknumber", Field.Track);
    public static final BlockDefS asul = new BlockDefS("asul", STR, "daap.songdataurl");
    public static final BlockDefN asur = new BlockDefN("asur", BYT, "daap.songuserrating");
    public static final BlockDefN asyr = new BlockDefN("asyr", SHO, "daap.songyear", Field.Year);
    public static final BlockDefN ated = new BlockDefN("ated", SHO, "daap.supportsextradata");
    public static final BlockDefL avdb = new BlockDefL("avdb", LIS, "daap.serverdatabases");
    public static final BlockDefN caar = new BlockDefN("caar", INT, "dacp.availablerepeatstates", false);
    public static final BlockDefN caas = new BlockDefN("caas", INT, "dacp.availableshufflestates", false);
    public static final BlockDefL caci = new BlockDefL("caci", LIS, "dacp.controlint", false);
    public static final BlockDefL cacr = new BlockDefL("cacr", LIS);    // new
    public static final BlockDefN caia = new BlockDefN("caia", BYT, "dacp.isavailable", false);
    public static final BlockDefS cana = new BlockDefS("cana", STR, "dacp.noplayingartist", false);
    public static final BlockDefS cang = new BlockDefS("cang", STR, "dacp.nowplayinggenre", false);
    public static final BlockDefS canl = new BlockDefS("canl", STR, "dacp.nowplayingalbum", false);
    public static final BlockDefS cann = new BlockDefS("cann", STR, "dacp.nowplayingname", false);
    public static final BlockDefS canp = new BlockDefS("canp", STR,  "dacp.nowplaying", null, false);    // really 4 ids
    public static final BlockDefN cant = new BlockDefN("cant", INT, "dacp.playingtime", false);
    public static final BlockDefN caps = new BlockDefN("caps", BYT, "dacp.playerstate", false);
    public static final BlockDefN carp = new BlockDefN("carp", BYT, "dacp.repeatstate", false);
    public static final BlockDefN cash = new BlockDefN("cash", BYT, "dacp.shufflestate", false);
    public static final BlockDefL casp = new BlockDefL("casp", LIS);
    public static final BlockDefN cass = new BlockDefN("cass", BYT, null, false);
    public static final BlockDefN cast = new BlockDefN("cast", INT, "dacp.songtime", false);
    public static final BlockDefN casu = new BlockDefN("casu", BYT, "dacp.casu", false);
    public static final BlockDefN cavc = new BlockDefN("cavc", BYT, "dacp.volumecontrollable");
    public static final BlockDefN ceJC = new BlockDefN("ceJC", BYS, "com.apple.itunes.jukebox-client-vote");
    public static final BlockDefN ceJI = new BlockDefN("ceJI", INT, "com.apple.itunes.jukebox-current");
    public static final BlockDefN ceJS = new BlockDefN("ceJS", SHS, "com.apple.itunes.jukebox-score");
    public static final BlockDefN ceJV = new BlockDefN("ceJV", INT, "com.apple.itunes.jukebox-vote");
    public static final BlockDefN ceSG = new BlockDefN("ceSG", BYT);
    public static final BlockDefL cmcp = new BlockDefL("cmcp", LIS, "dmcp.controlprompt", false);
    public static final BlockDefL cmgt = new BlockDefL("cmgt", LIS, "dmcp.getpropertyresponse", false);
    public static final BlockDefN cmik = new BlockDefN("cmik", BYT);
    public static final BlockDefN cmmk = new BlockDefN("cmmk", INT, "dmcp.mediakind");
    public static final BlockDefL cmpa = new BlockDefL("cmpa", LIS);  // Used when pairing
    public static final BlockDefN cmpg = new BlockDefN("cmpg", LON);  // Used when pairing
    public static final BlockDefS cmnm = new BlockDefS("cmnm", STR);  // Used when pairing
    public static final BlockDefN cmsp = new BlockDefN("cmsp", BYT);  // 
    public static final BlockDefN cmsr = new BlockDefN("cmsr", INT, "dmcp.serverrevision", false);
    public static final BlockDefL cmst = new BlockDefL("cmst", LIS, "dmcp.status", false);
    public static final BlockDefN cmsv = new BlockDefN("cmsv", BYT);
    public static final BlockDefS cmty = new BlockDefS("cmty", STR); // Used when pairing
    public static final BlockDefN cmvo = new BlockDefN("cmvo", INT, "dmcp.volume", false);
    public static final BlockDefN fuch = new BlockDefN("f\u008dch", BYT, "dmap.haschildcontainers");
    public static final BlockDefL mbcl = new BlockDefL("mbcl", LIS, "dmap.bag");
    public static final BlockDefL mccr = new BlockDefL("mccr", LIS, "dmap.contentcodesresponse");
    public static final BlockDefS mcna = new BlockDefS("mcna", STR, "dmap.contentcodesname");
    public static final BlockDefN mcnm = new BlockDefN("mcnm", INT, "dmap.contentcodesnumber");
    public static final BlockDefL mcon = new BlockDefL("mcon", LIS, "dmap.container");
    public static final BlockDefN mctc = new BlockDefN("mctc", INT, "dmap.containercount");
    public static final BlockDefN mcti = new BlockDefN("mcti", INT, "dmap.containeritemid");
    public static final BlockDefN mcty = new BlockDefN("mcty", SHO, "dmap.contentcodestype");
    public static final BlockDefL mdcl = new BlockDefL("mdcl", LIS, "dmap.dictionary");
    public static final BlockDefL medc = new BlockDefL("medc", LIS, "dmap.editdictionary", false);
    public static final BlockDefN meds = new BlockDefN("meds", INT, "dmap.editcommandssupported");
    public static final BlockDefN miid = new BlockDefN("miid", INT, "dmap.itemid", DaapField.DatabaseIndex);
    public static final BlockDefN mikd = new BlockDefN("mikd", BYT, "dmap.itemkind", DaapField.ItemKind);
    public static final BlockDefN mimc = new BlockDefN("mimc", INT, "dmap.itemcount");
    public static final BlockDefS minm = new BlockDefS("minm", STR, "dmap.itemname", Field.Title);
    public static final BlockDefL mlcl = new BlockDefL("mlcl", LIS, "dmap.listing");
    public static final BlockDefN mlid = new BlockDefN("mlid", INT, "dmap.sessionid");
    public static final BlockDefL mlit = new BlockDefL("mlit", LIS, "dmap.listingitem");
    public static final BlockDefL mlog = new BlockDefL("mlog", LIS, "dmap.loginresponse");
    public static final BlockDefN mpco = new BlockDefN("mpco", INT, "dmap.parentcontainerid");
    public static final BlockDefN mper = new BlockDefN("mper", LON, "dmap.persistentid");
    public static final BlockDefN mpro = new BlockDefN("mpro", VER, "dmap.protocolversion");
    public static final BlockDefN mrco = new BlockDefN("mrco", INT, "dmap.returnedcount");
    public static final BlockDefN msal = new BlockDefN("msal", BYT, "dmap.supportsautologout");
    public static final BlockDefN msas = new BlockDefN("msas", BYT, "dmap.authenticationschemes");
    public static final BlockDefN msau = new BlockDefN("msau", BYT, "dmap.authenticationmethod");
    public static final BlockDefN msbr = new BlockDefN("msbr", BYT, "dmap.supportsbrowse");
    public static final BlockDefN msdc = new BlockDefN("msdc", INT, "dmap.databasescount");
    public static final BlockDefN msed = new BlockDefN("msed", BYT, "daap.supportsedit", false);
    public static final BlockDefN msex = new BlockDefN("msex", BYT, "dmap.supportsextensions");
    public static final BlockDefN mshc = new BlockDefN("mshc", SHO);    // new
    public static final BlockDefN mshi = new BlockDefN("mshi", INT);    // new
    public static final BlockDefL mshl = new BlockDefL("mshl", LIS);    // new
    public static final BlockDefN mshn = new BlockDefN("mshn", INT);    // new
    public static final BlockDefN msix = new BlockDefN("msix", BYT, "dmap.supportsindex");
    public static final BlockDefN mslr = new BlockDefN("mslr", BYT, "dmap.loginrequired");
    public static final BlockDefN msma = new BlockDefN("msma", LON);
    public static final BlockDefL msml = new BlockDefL("msml", LIS, "dmap.speakermachinelist", false);
    public static final BlockDefN mspi = new BlockDefN("mspi", BYT, "dmap.supportspersistentids");
    public static final BlockDefN msqy = new BlockDefN("msqy", BYT, "dmap.supportsquery");
    public static final BlockDefN msrs = new BlockDefN("msrs", BYT, "dmap.supportsresolve");
    public static final BlockDefL msrv = new BlockDefL("msrv", LIS, "dmap.serverinforesponse");
    public static final BlockDefN mstc = new BlockDefN("mstc", DAT, "dmap.utctime");
    public static final BlockDefN mstm = new BlockDefN("mstm", INT, "dmap.timeoutinterval");
    public static final BlockDefN msto = new BlockDefN("msto", INS, "dmap.utcoffset");
    public static final BlockDefS msts = new BlockDefS("msts", STR, "dmap.statusstring");
    public static final BlockDefN mstt = new BlockDefN("mstt", INT, "dmap.status");
    public static final BlockDefN msup = new BlockDefN("msup", BYT, "dmap.supportsupdate");
    public static final BlockDefN mtco = new BlockDefN("mtco", INT, "dmap.specifiedtotalcount");
    public static final BlockDefL mudl = new BlockDefL("mudl", LIS, "dmap.deletedidlisting");
    public static final BlockDefL mupd = new BlockDefL("mupd", LIS, "dmap.updateresponse");
    public static final BlockDefN musr = new BlockDefN("musr", INT, "dmap.serverrevision");
    public static final BlockDefN muty = new BlockDefN("muty", BYT, "dmap.updatetype");

    public static final BlockDefL ceWM = new BlockDefL("ceWM", LIS);
    public static final BlockDefN ceVO = new BlockDefN("ceVO", BYT);

    /**
     * Class representing a Block definitiion
     */
    static class BlockDef {

        private final int type;
        private final String name, desc;
        private final AbstractField<?> field;

        BlockDef(String name, int type, String desc, AbstractField<?> field, boolean content) {
            this.name = name;
            this.type = type;
            this.desc = desc;
            this.field = field;
            defmap.put(new Integer(getCode()), this);
            if (desc!=null) {
                defmap.put(desc, this);
            }
            if (desc!=null && content) {
                contentcodes.add(this);
            }
        }

        public String getName() {
            return name;
        }

        public int getCode() {
            return (name.charAt(0)<<24) + (name.charAt(1)<<16) + (name.charAt(2)<<8) + (name.charAt(3)<<0);
        }

        public final int hashCode() {
            return getCode();
        }

        public AbstractField<?> getField() {
            return field;
        }

        public String getDescription() {
            return desc;
        }

        public int getType() {
            return type;
        }

        public String toString() {
            return name;
        }

    }

    /**
     * Class representing a Block Definition for a NumericBlock
     */
    static class BlockDefN extends BlockDef {
        BlockDefN(String name, int type, String desc, AbstractField field, boolean report) {
            super(name, type, desc, field, report);
            if (type==STR || type==LIS) throw new Error();
        }
        BlockDefN(String name, int type, String desc, AbstractField field) {
            this(name, type, desc, field, true);
        }
        BlockDefN(String name, int type, String desc, boolean report) {
            this(name, type, desc, null, report);
        }
        BlockDefN(String name, int type) {
            this(name, type, null, null, false);
        }
        BlockDefN(String name, int type, String desc) {
            this(name, type, desc, null, false);
        }
    }

    /**
     * Class representing a Block Definition for a StringBlock
     */
    static class BlockDefS extends BlockDef {
        BlockDefS(String name, int type, String desc, AbstractField field, boolean report) {
            super(name, type, desc, field, report);
            if (type!=STR) throw new Error();
        }
        BlockDefS(String name, int type, String desc, AbstractField field) {
            this(name, type, desc, field, true);
        }
        BlockDefS(String name, int type, String desc, boolean report) {
            this(name, type, desc, null, report);
        }
        BlockDefS(String name, int type) {
            this(name, type, null, null, false);
        }
        BlockDefS(String name, int type, String desc) {
            this(name, type, desc, null, false);
        }
    }

    /**
     * Class representing a Block Definition for a ListBlock
     */
    static class BlockDefL extends BlockDef {
        BlockDefL(String name, int type, String desc, boolean report) {
            super(name, type, desc, null, true);
        }
        BlockDefL(String name, int type) {
            this(name, type, null, false);
        }
        BlockDefL(String name, int type, String desc) {
            this(name, type, desc, true);
        }
    }

    /**
     * Create a new NumericBlock
     */
    public static NumericBlock create(BlockDefN def, long value) {
        NumericBlock block = new NumericBlock();
        block.setCode(def.getCode());
        block.setType(def.getType());
        block.set(value);
        return block;
    }

    /**
     * Create a new NumericBlock
     */
    public static StringBlock create(BlockDefS def, String value) {
        StringBlock block = new StringBlock();
        block.setCode(def.getCode());
        block.set(value);
        return block;
    }

    /**
     * Create a new NumericBlock
     */
    public static StringBlock create(BlockDefS def, byte[] value) {
        StringBlock block = new StringBlock();
        block.setCode(def.getCode());
        block.set(value);
        return block;
    }

    /**
     * Create a new ListBlock
     */
    public static ListBlock create(BlockDefL def) {
        ListBlock block = new ListBlock();
        block.setCode(def.getCode());
        return block;
    }

    /**
     * Create a new StringBlock
     */
    public static StringBlock createString(BlockDef def, String value) {
        StringBlock block = new StringBlock();
        block.setCode(def.getCode());
        block.set(value);
        return block;
    }

    /**
     * Create a new StringBlock
     */
    public static ListBlock createList(BlockDef def, String value) {
        ListBlock block = new ListBlock();
        block.setCode(def.getCode());
        return block;
    }

    /**
     * Create a new StringBlock
     */
    public static NumericBlock createNumeric(BlockDef def, int type, long value) {
        NumericBlock block = new NumericBlock();
        block.setType(type);
        block.set(value);
        return block;
    }

    /**
     * Create a new Block of the specified Type
     */
    public static Block createTyped(BlockDef def, Object value) {
        if (def instanceof BlockDefS) {
            return create((BlockDefS)def, value.toString());
        } else {
            return create((BlockDefN)def, value==Boolean.TRUE ? 1 : value==Boolean.FALSE ? 0 : value instanceof Number ? ((Number)value).longValue() : Long.parseLong((String)value));
        }
    }

    /**
     * Get the Block Definition from the long description, eg "dmap.itemkind"
     */
    static BlockDef getFromDescription(String desc) {
        return (BlockDef)defmap.get(desc);
    }

    private static BlockDef getFromCode(int code) {
        return (BlockDef)defmap.get(new Integer(code));
    }

    static String codeToName(int code) {
        try {
            byte[] c = new byte[] { (byte)(code>>24), (byte)(code>>16), (byte)(code>>8), (byte)code };
            return new String(c, "ISO-8859-1");
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static int readInt(InputStream in) throws IOException {
        int c1 = in.read();
        int c2 = in.read();
        int c3 = in.read();
        int c4 = in.read();
        if (c4 < 0) throw new EOFException();
        return (c1<<24) + (c2<<16) + (c3<<8) + c4;
    }

    /**
     * Read a Block from the InputStream. Not actually used in the current codebase
     * as we only write blocks as a DACP/DAAP Server. Used for debugging however.
     * @param in the InputStream
     * @param parent - the Parent ListBlock, or null
     */
    static Block read(InputStream in, ListBlock parent) throws IOException {
        int code = readInt(in);
        int len = readInt(in);
        Block block;
        BlockDef def = getFromCode(code);
        if (def==null) {
            throw new IOException("Unknown block \""+codeToName(code)+"\" len="+len);
        } else if (def.getType()==STR) {
            block = new StringBlock();
        } else if (def.getType()==LIS) {
            int parcode = parent==null ? 0 : parent.getCode();
            if (code == mlit.getCode() && (parcode==abar.getCode() || parcode==abgn.getCode() || parcode==abcp.getCode() || parcode==abal.getCode())) {
                block = new StringBlock();
            } else {
                block = new ListBlock();
            }
        } else {
            block = new NumericBlock();
            ((NumericBlock)block).setType(def.getType());
            if (len!=block.length()) {
                throw new IOException("Bad length for \""+codeToName(code)+"\": expected "+block.length()+" got "+len);
            }
        } 
        block.setCode(code);
        block.read(in, len);
        // Sanity check, not required so far
        int readlen = block.length();
        if (readlen < len) {
            System.out.println("WARNING: Skipping "+(len-readlen)+" bytes");
            long amount;
            while (readlen < len && (amount=in.skip(len - readlen)) > 0) {
                readlen -= amount;
            }
        }
        if (readlen != len) {
            throw new Error("Error: readlen="+readlen+" len="+len);
        }
        return block;
    }

    /**
     * Return the List of content-codes, as created by the Block Definition constructors
     */
    static List<BlockDef> getContentCodes() {
        return contentcodes;
    }

    /**
     * Get the value of the  block for this description
     * @param desc the long description, eg "dmap.itemkind"
     * @param track the Track
     */
    static Block getFieldAsBlock(String desc, Track track) {
        Blocks.BlockDef def = Blocks.getFromDescription(desc);
        if (def!=null) {
            AbstractField<?> field = def.getField();
            Object val = field==null ? null : track.get(field);
            if (val!=null) {
                return Blocks.createTyped(def, val);
            }
        }
        return null;
    }

    /**
     * Return the list of long-descriptions that should be used when "meta=all"
     * is requested. This is only fdone for higher revisions of the protocol
     */
    static String[] getAllMeta() {
        List<String> descs = new ArrayList<String>();
        for (Iterator<BlockDef> i = defmap.values().iterator();i.hasNext();) {
            BlockDef def = i.next();
            if (def.getField()!=null && def.getDescription()!=null) {
                descs.add(def.getDescription());
            }
        }
        return descs.toArray(new String[0]);
    }

}
