package org.qtunes.daap;

import org.qtunes.db.*;
import org.qtunes.web.*;
import java.util.*;
import java.io.*;

class HandlerGroups extends AbstractHandler {

    Block handleDAAP(DaapServer daap, WebConnection con) throws IOException {
        String[] meta = (con.getParameter("meta")).split(",");
        String mediatype = con.getParameter("type");
        String grouptype = con.getParameter("group-type");
        String sort = con.getParameter("sort");
        boolean includesortheaders = "1".equals(con.getParameter("include-sort-headers"));
        Database db = daap.getDatabase();

        List<Group> groups = daap.getGroupList(DaapPlaylist.getSortedSmartPlaylist(con.getParameter("query"), db, sort));
        ListBlock agal = Blocks.create(Blocks.agal);
        agal.add(Blocks.create(Blocks.mstt, 200));
        agal.add(Blocks.create(Blocks.muty, 0));

        ListBlock mlcl = Blocks.create(Blocks.mlcl);
        ListBlock mshl = Blocks.create(Blocks.mshl);
        boolean sortalbum = "album".equals(sort);
        boolean sortartist = "artist".equals(sort);

        int lastsortchar = -1, lastsortindex = 0;
        for (int i=0;i<groups.size();i++) {
            Group group = groups.get(i);
            ListBlock mlit = Blocks.create(Blocks.mlit);
            mlcl.add(mlit);
            String album;
            mlit.add(Blocks.create(Blocks.miid, group.getIndex()));
            mlit.add(Blocks.create(Blocks.mper, group.getPersistentId()));
            mlit.add(Blocks.create(Blocks.minm, album = group.getAlbum()));
            String artist = group.getArtist();
            if (artist!=null) {
                mlit.add(Blocks.create(Blocks.asaa, artist));
            }
            mlit.add(Blocks.create(Blocks.mimc, group.getTracks().cardinality()));
            char sortchar = getSortChar(sortalbum ? album : artist);
            if ((sortchar>lastsortchar || sortchar=='0' && lastsortchar <='Z') || i==groups.size()-1) {
                if (i!=0) {
                    mlit = Blocks.create(Blocks.mlit);
                    mshl.add(mlit);
                    mlit.add(Blocks.create(Blocks.mshc, lastsortchar));         // char: 'A'..'Z', '0' or 0
                    mlit.add(Blocks.create(Blocks.mshi, lastsortindex));        // first track for this char
                    mlit.add(Blocks.create(Blocks.mshn, i - lastsortindex));    // num tracks for this char
                }
                lastsortindex = i;
                lastsortchar = sortchar;
            }
        }

        agal.add(Blocks.create(Blocks.mtco, mlcl.size()));
        agal.add(Blocks.create(Blocks.mrco, mlcl.size()));
        agal.add(mlcl);
        if (includesortheaders) {
            agal.add(mshl);
        }
        return agal;
    }

    private static char getSortChar(String s) {
        if (s==null || s.length()==0) {
            return '0';
        } else {
            try {
                s = java.text.Normalizer.normalize(s, java.text.Normalizer.Form.NFKD);  // Java16
            } catch (Throwable e) {}
            s = s.trim().toUpperCase();
            if (s.startsWith("THE ")) {
                s = s.substring(4);
            }
            for (int i=0;i<s.length();i++) {
                char c = s.charAt(i);
                if (c>='A' && c<='Z') {
                    return c;
                } else if (c>='0' && c<='9') {
                    return '0';
                }
            }
            return '0';
        }
    }
}
