package org.qtunes.db;

/**
 * A DerivedField is a field derived from other properties of the Track.
 * These are mainly used for Services that need them, eg the {@link org.qtunes.daap.DaapServer}
 * creates a few in the {@link org.qtunes.daap.DaapField} class
 */
public abstract class DerivedField<T> implements AbstractField<T> {

    private final String name;

    public DerivedField(String name) {
        this.name = name;
        Field.register(this);
    }

    public abstract T get(Track track);

    /**
     * Can this derived value be cached, or will it change over time?
     */
    public boolean isCacheable() {
        return false;
    }

    public String getName() {
        return name;
    }

    public String toString() {
        return name;
    }

}
