package org.qtunes.db;

import org.qtunes.core.*;
import java.util.*;
import java.io.*;

/**
 * A Playlist which determines it's tracks based on a filter. The
 * filter uses a {@link TrackMatcher}, so you can for example do
 * <code>Album=~"Buzzin' Fly.*"</code> or <code>(Artist="Postal Service" or
 * artist=~"Death Cab.*") and Year > "2000"</code>. The "sort" key can be
 * set to a comma-seperated list of fields to change the order, eg:
 * <code>artist="Chicane" and sort="album,disc,track"</code>. For each
 * field the prefix "!" can be used to reverse it, eg <code>sort="!lastmodified"</code>
 * to get oldest tracks first.
 */
public interface SmartPlaylist extends Playlist {

    /**
     * Return the Query used to determine the Playlist,
     * The format of which depends on the implementation
     */
    public String getQuery();

    /**
     * Set the query string used to determine the Playlist. The format
     * depends on the implementation.
     */
    public void setQuery(String query);

}
