package org.qtunes.ff;

import org.qtunes.db.*;
import java.util.*;
import java.io.*;

/**
 * An interface which can be implemented by a {@link TrackReader}
 * if it can update a File with new fields
 */
public abstract class TrackWriter extends TrackReader {

    /**
     * Update the track
     * @param track the Track with the existing fields
     * @param changes the Changes to those fields - a value of <code>null</code>
     * in this Map means the field is deleted, and a key of
     * {@link Field#LastModified} means the File modification time will not be updated.
     */
    public abstract void write(Track track, FieldMap changes) throws IOException;

    /**
     * Update a file directly - can be used by utility classes (eg tagger package)
     * to update files without a database.
     * @param file the audio File to update
     * @param changes the Changes to those fields - a value of <code>null</code>
     * in this Map means the field is deleted, and a key of
     * {@link Field#LastModified} means the File modification time will not be updated.
     */
    public abstract boolean write(File file, FieldMap changes) throws IOException;

}
