package org.qtunes.ff.spi.mp3;

import java.nio.*;
import java.io.*;
import java.util.*;

class TagComment extends Tag {

    private String lang, type, value;

    TagComment() {
    }

    TagComment(int version, int flags, String lang, String type, String value) {
        super(version==2 ? "COM" : "COMM", version, flags, encode(version, lang, type, value));
    }

    private static byte[] encode(int version, String lang, String type, String value) {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            int encoding = getStringEncoding(version, type+value);
            out.write(encoding);
            out.write(lang.charAt(0));
            out.write(lang.charAt(1));
            out.write(lang.charAt(2));
            writeString(encoding, type, out);
            writeStringTerminator(encoding, out);
            writeString(encoding, value, out);
            writeStringTerminator(encoding, out);       // Not strictly required, but iTunes does
            return out.toByteArray();
        } catch (IOException e) { 
            throw new RuntimeException(e);
        }
    }

    private synchronized void parse() throws IOException {
        if (value==null) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.get();
                lang = readFixedString(data, 3);
                type = readEncodedString(data, encoding);
                value = readEncodedString(data, encoding);
            } catch (BufferUnderflowException e) {
                throw new IOException(e);
            }
        }
    }

    String getLanguage() throws IOException {
        parse();
        return lang;
    }

    String getType() throws IOException {
        parse();
        return type;
    }

    String getValue() throws IOException {
        parse();
        return value;
    }

    public String toString() {
        try {
            return "["+getName()+" lang="+getLanguage()+" type="+getType()+" value="+getValue()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

}
