package org.qtunes.ff.spi.mp3;

import java.nio.*;
import java.io.*;

class TagTypedText extends TagText {

    private String lang, type, value;

    TagTypedText() {
    }

    TagTypedText(String name, int version, int flags, String type, String value) {
        super(name, version, flags, encode(version, type, value));
    }

    private static byte[] encode(int version, String type, String value) {
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            int encoding = getStringEncoding(version, type+value);
            out.write(encoding);
            writeString(encoding, type, out);
            writeStringTerminator(encoding, out);
            writeString(encoding, value, out);
            return out.toByteArray();
        } catch (IOException e) { 
            throw new RuntimeException(e);
        }
    }

    private synchronized void parse() throws IOException {
        if (value==null) {
            try {
                ByteBuffer data = getContent();
                data.rewind();
                byte encoding = data.get();
                type = readEncodedString(data, encoding);
                value = readEncodedString(data, encoding);
            } catch (BufferUnderflowException e) {
                throw new IOException(e);
            }
        }
    }

    String getType() throws IOException {
        parse();
        return type;
    }

    String getValue() throws IOException {
        parse();
        return value;
    }

    public String toString() {
        try {
            return "["+getName()+" type="+getType()+" value="+getValue()+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

}
