package org.qtunes.player.spi;

import org.qtunes.player.*;
import org.qtunes.db.*;
import org.qtunes.core.*;
import org.qtunes.auth.*;
import javax.sound.sampled.AudioFormat;

class TrackSession {

    final Track track;
    final Session session;
    private volatile int offset;
    private AudioFormat format;
    private float bpms;         // bytes per MS

    TrackSession(Track track, Session session) {
        this.track = track;
        this.session = session;
    }

    AudioFormat getFormat() {
        return format;
    }

    boolean formatMatches(TrackSession session) {
        return session!=null && session.format.matches(format);
    }

    void setFormat(AudioFormat format) {
        if (this.format!=null && !this.format.matches(format)) {
            throw new Error();
        }
        this.format = format;
        this.bpms = format.getSampleRate() * format.getSampleSizeInBits() * format.getChannels() / 8000;
        this.offset = 0;
    }

    public String toString() {
        return "[track="+track.getIndex()+" session="+session.getId()+" offset="+offset+"]";
    }

    void advance(int delta) {
        this.offset += delta;
    }

    void seek(int offset) {
        this.offset = offset;
    }

    int tell() {
        return offset;
    }

    void seekMS(int ms) {
        int bytes = (int)(ms * bpms) & ~3;
        int length = (int)(track.get(Field.Duration) * bpms) & ~3;
        if (bytes >= 0 && bytes <= length) {
            seek(bytes);
        }
    }

    int tellMS() {
        return (int)(offset / bpms);
    }

}
