package org.qtunes.replaygain;

import org.qtunes.core.util.Base64;
import java.util.zip.*;
import java.util.*;
import java.io.*;

/**
 * Represents a previously calculated analysis, which may be saved to
 * a database or passed into {@link #getAdjustment}
 */
public class GainAnalysis implements java.io.Serializable {

    static final GainAnalysis FAILED = new GainAnalysis((byte[])null);
    private static final long serialVersionUID = -2885507831895088255l;
    private byte[] data;

    private GainAnalysis(byte[] data) {
        this.data = data;
    }

    GainAnalysis(int[] data) {
        this.data = compress(data);
    }

    public boolean equals(Object o) {
        return o instanceof GainAnalysis ? data == ((GainAnalysis)o).data : false;
    }

    int[] getData() {
        return decompress(data);
    }

    private static byte[] compress(int[] in) {
        try {
            int m = 0, n = 1;
            for (int i=0;i<in.length;i++) {
                m |= in[i];
            }
            while (m > 0) {
               n++;
               m>>=1;
            }
            ByteArrayOutputStream bout = new ByteArrayOutputStream();
            GZIPOutputStream out = new GZIPOutputStream(bout);
            out.write(in.length>>16);
            out.write(in.length>>8);
            out.write(in.length);
            int outlength = ((in.length*n+7)>>3)+1;
            int j = 0, s = 0, a = 0;
            int pad = ((outlength-1)*8) - (in.length*n);
            out.write(((n-1)<<3) | pad);
            for (int i=0;i<in.length;i++) {
                a = a<<n | in[i];
                s += n;
                while (s > 7) {
                    s -= 8;
                    out.write((a>>s) & 0xFF);
                }
            }
            if (s > 0) {
                out.write(a<<(8-s));
            }
            out.close();
            return bout.toByteArray();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private static int[] decompress(byte[] din) {
        try {
            GZIPInputStream in = new GZIPInputStream(new ByteArrayInputStream(din));
            int numsamples = (in.read() << 16) | (in.read() << 8) | in.read();
            if (numsamples < 0) throw new EOFException();
            int c = in.read();
            if (c<0) throw new EOFException();
            int n = (c >> 3) + 1;
            int pad = 8 + (c & 7);
            int j = 0, s = 0, a = 0;
            int[] out = new int[numsamples];
            while (j < numsamples && (c=in.read())>=0) {
                a = (a<<8) | c;
                s += 8;
                while (s >= n && j < numsamples) {
                    out[j++] = a>>(s-n) & ((1<<n)-1);
                    s -= n;
                }
            }
            return out;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Get the adjustment required to set the track to the specified Gain level
     */
    public float getAdjustment(float gain) {
        return ReplayGainAnalyzer.getAdjustment(this.getData(), gain);
    }

    /**
     * Get the adjustment required to set the specified tracks to the specified Gain level
     */
    public static float getAdjustment(Collection<GainAnalysis> analyses, float gain) {
        int[] album = null;
        for (Iterator<GainAnalysis> i = analyses.iterator();i.hasNext();) {
            int[] track = i.next().getData();
            if (album==null) {
                album = track;
            } else {
                for (int j=0;j<track.length;j++) {
                    album[j] += track[j];
                }
            }
        }
        return ReplayGainAnalyzer.getAdjustment(album, gain);
    }

    public String toString() {
        return data==null ? "" : Base64.encode(data);
    }
    
    public static GainAnalysis fromString(String data) {
        return data==null || data.length() == 0 ? FAILED : new GainAnalysis(Base64.decode(data));
    }

}
