package org.qtunes.tagger;

import org.qtunes.core.*;
import org.qtunes.ff.*;
import org.qtunes.db.*;
import java.io.*;
import java.util.*;
import java.nio.*;
import java.nio.channels.*;
import javax.sound.sampled.*;
import org.qtunes.core.util.FileUtils;

public class Main {

    private static final String VERSION = "1.0";
    private static PrintStream out = System.out;
    private static boolean fatal = false;

    public static void main(String[] args) throws Exception {

        Action action = null;

        if (args.length == 0) {
            fail();
        } else if (args[0].equals("list")) {
            action = new Lister();
        } else if (args[0].equals("edit")) {
            action = new Tagger();
        } else if (args[0].equals("script")) {
            action = new Scripter();
        } else if (args[0].equals("normalize")) {
            action = new NormalizerTool();
        } else {
            fail();
        }

        if (action != null) {
            int i;
            File filelist = null;
            for (i=1;i<args.length;i++) {
                if (args[i].equals("--output")) {
                    action.setOutput(out = new PrintStream(new FileOutputStream(args[++i]), true, "UTF-8"));
                } else if (args[i].equals("--fatal")) {
                    fatal = true;
                } else if (args[i].equals("--debug")) {
                    action.bumpDebug();
                } else if (args[i].equals("--root")) {
                    FileUtils.setRoot(new File(args[++i]));
                } else if (args[i].equals("--filelist")) {
                    if (filelist == null) {
                        filelist = new File(args[++i]);
                    } else {
                        fail();
                    }
                } else if (args[i].equals("--")) {
                    break;
                } else if (args[i].startsWith("--") && action instanceof Lister) {
                    fail();
                } else if (args[i].startsWith("--") && action instanceof Tagger) {
                    if (args[i].equals("--dryrun")) {
                        ((Tagger)action).setDryRun(true);
                    } else if (args[i].equals("--set")) {
                        String s = args[++i];
                        int ix = s.indexOf("=");
                        if (ix < 0) {
                            fail();
                        } else {
                            ((Tagger)action).setTag(s.substring(0, ix), s.substring(ix+1));
                        }
                    } else {
                        fail();
                    }
                } else if (args[i].startsWith("--") && action instanceof Scripter) {
                    if (args[i].equals("--dryrun")) {
                        ((Scripter)action).setDryRun(true);
                    } else if (args[i].equals("--norename")) {
                        ((Scripter)action).setPrintRename(true);
                    } else if (args[i].equals("--ci")) {
                        ((Scripter)action).setCaseInsensitive(true);
                    } else if (args[i].equals("--scriptfile")) {
                        Reader reader = new InputStreamReader(new FileInputStream(args[++i]), "UTF-8");
                        ((Scripter)action).setScript(reader);
                        reader.close();
                    } else if (args[i].equals("--function")) {
                        ((Scripter)action).setFunction(args[++i]);
                    } else {
                        fail();
                    }
                } else if (args[i].startsWith("--") && action instanceof NormalizerTool) {
                    if (args[i].equals("--dryrun")) {
                        ((NormalizerTool)action).setDryRun(true);
                    } else if (args[i].equals("--db")) {
                        ((NormalizerTool)action).setDatabase(new File(args[++i]));
                    } else if (args[i].equals("--track") || args[i].equals("--album") || args[i].equals("--auto")) {
                        ((NormalizerTool)action).setMode(args[i].substring(2), Float.parseFloat(args[++i]));
                    } else if (args[i].equals("--remove")) {
                        ((NormalizerTool)action).setMode("remove", 0);
                    } else if (args[i].equals("--tag")) {
                        ((NormalizerTool)action).setTagType(args[++i]);
                    } else {
                        fail();
                    }
                } else if (args[i].startsWith("--")) {
                    fail();
                } else {
                    if (filelist == null) {
                        process(action, new File(args[i]));
                    } else {
                        fail();
                    }
                }
            }

            if (filelist != null) {
                if (i != args.length) {
                    fail();
                }
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(filelist), "UTF-8"));
                String s;
                while ((s=reader.readLine())!=null) {
                    process(action, new File(s));
                }
                reader.close();
            }
            for (;i<args.length;i++) {
                process(action, new File(args[i]));
            }

            action.close();
        }
    }

    private static void fail() {
        System.out.println("Tagger v"+VERSION+"    http://qtunes.org");
        System.out.println("Usage: Tagger [list | edit | normalize] args file...\n");
        System.out.println();
        System.out.println("    --output file        the File to write the output to");
        System.out.println("    --debug              increase debugging level");
        System.out.println("    --fatal              errors on individual track stop execution");
        System.out.println("    --root path          all music files are relative to the specified path");
        System.out.println("    --filelist file      take the list of files to process from 'file' instead");
        System.out.println();

        System.out.println("  list");
        System.out.println();

        System.out.println("  edit");
        System.out.println("    --dryrun             report on changes, but don't make them");
        System.out.println("    --set field=value    to set a tag to the specified value, or if value");
        System.out.println();
        System.out.println("  Fields are: title, artist, album, alvumartist, genre, track, numtracks, disc,");
        System.out.println("              numdiscs, composer, year, compilation, comment, cddb, mcdi,");
        System.out.println("              encodersettings, group, language, publisher, url, bpm, userinfo,");
        System.out.println("              isrc, media, copyright, originalartist, originalalbum, conducter,");
        System.out.println("              lyricist, type, file, bitrate, samplerate, mono, vbr, duration,");
        System.out.println("              filesize, added, lastmodified, gain, gapless, id3v1, id3v22,");
        System.out.println("              id3v23, id3v24, warning, imagedescription, imagemimetype,");
        System.out.println("              imageoffset, imagelength, audiobyterange.");
        System.out.println();
        System.out.println("  script");
        System.out.println("    --dryrun             report on changes, but don't make them");
        System.out.println("    --norename           don't rename files - print \"mv\" operations instead");
        System.out.println("    --ci                 changes in filename case are ignored");
        System.out.println("    --scriptfile file    File to load the JavaScript from");
        System.out.println("    --function file      function-name to pass in to methods");
        System.out.println();
        System.out.println("    JavaScript file should define methods before(function), after(function)");
        System.out.println("    and action(function, track), which will be called with the function name.");
        System.out.println("    Action will be called once for each track with a 'track' object containing");
        System.out.println("    the fields above. Tracks can be renamed by changing 'track.file', and the image");
        System.out.println("    can be accessed as a byte array by reading 'track.image'. Any changes to other");
        System.out.println("    track fields will cause the file to be updated");
        System.out.println();
        System.out.println("  normalize");
        System.out.println("    --db file            the file to save the calculated results to");
        System.out.println("    --track gain         set each track to have the specified gain");
        System.out.println("    --album gain         set all tracks to average to the specified gain");
        System.out.println("    --auto gain          gapless albums will be set to average to the specified");
        System.out.println("                         gain, all other tracks will be set as for '--track'");
        System.out.println("    --remove             remove the specified tracks from the database");
        System.out.println("    --tag [itunes,replaygain,rva]  set the type of normalization tag to write");

        System.out.println();
        System.exit(0);
    }

    private static void process(Action action, File file) throws Exception {
        File absolutefile = FileUtils.getAbsoluteFile(file);
        if (absolutefile.isDirectory()) {
            File[] files = absolutefile.listFiles();
            Arrays.sort(files);
            for (int i=0;i<files.length;i++) {
                process(action, files[i]);
            }
        } else if (absolutefile.canRead() && !absolutefile.getPath().contains(".AppleDouble")) {
            try {
                action.process(FileUtils.getRelativeFile(absolutefile));
            } catch (Exception e) {
                if (fatal) {
                    throw e;
                } else {
                    out.println("# Failed on \""+absolutefile+"\"");
                    e.printStackTrace(out);
                }
            }
        }
    }

}
