package org.qtunes.tagger;

import org.qtunes.core.*;
import org.qtunes.ff.*;
import org.qtunes.db.*;
import java.io.*;
import java.util.*;
import java.text.*;
import java.text.Normalizer;
import java.nio.*;
import java.nio.channels.*;
import javax.sound.sampled.*;

class Tagger implements Action {

    private PrintStream out = System.out;
    private FieldMap changes = new FieldMap();
    private int debug;
    private boolean dryrun;

    public void setOutput(PrintStream out) {
        this.out = out;
    }

    public void setDryRun(boolean dryrun) {
        this.dryrun = dryrun;
    }

    public void bumpDebug() {
        this.debug++;
    }

    private static Object cast(Field f, Object o) {
        if (o == null) {
            return o;
        } else if (f.getType() == String.class) {
            return o.toString();
        } else if (f.getType() == Integer.class) {
            if (o instanceof Boolean) {
                return Integer.valueOf(o == Boolean.TRUE ? 1 : 0);
            } else if (o instanceof Integer) {
                return o;
            } else if (o instanceof Number) {
                return Integer.valueOf(((Number)o).intValue());
            } else if (o instanceof String) {
                return Integer.valueOf((String)o);
            }
        } else if (f.getType() == Float.class) {
            if (o instanceof Float) {
                return o;
            } else if (o instanceof Number) {
                return Float.valueOf(((Number)o).floatValue());
            } else if (o instanceof String) {
                return Float.valueOf((String)o);
            }
        } else if (f.getType() == Long.class) {
            if (o instanceof Number) {
                return Long.valueOf(((Number)o).longValue());
            } else if (o instanceof String) {
                return Long.valueOf((String)o);
            }
        } else if (f.getType() == Boolean.class) {
            if (o instanceof Boolean ) {
                return o;
            } else if (o instanceof Number) {
                int v = ((Number)o).intValue();
                return v==0 ? Boolean.FALSE : Boolean.TRUE;
            } else if (o instanceof String) {
                return Boolean.valueOf((String)o);
            }
        }
        throw new ClassCastException("Can't convert "+o.getClass().getName()+" "+o+" to "+f.getType().getName()+" for '"+f.getName().toLowerCase()+"'");
    }

    void setTag(String key, String value) {
        AbstractField field = Field.getByName(key);
        if (field instanceof Field) {
            if (value == null || value.length() == 0) {
                changes.putUntyped((Field)field, null);
            } else {
                changes.putUntyped((Field)field, cast((Field)field, value));
            }
        } else {
            throw new IllegalArgumentException("Unknown field \""+key+"\"");
        }
    }

    public void process(File file) throws IOException {
        TrackReader reader = TrackReader.getTrackReader(file);
        if (reader instanceof TrackWriter) {
            FieldMap map = reader.read(file);
            boolean changed = map.get(Field.Warning) != null;
            for (Iterator<Map.Entry<Field<?>,Object>> i = changes.entrySet().iterator();i.hasNext();) {
                Map.Entry<Field<?>,Object> e = i.next();
                Field field = e.getKey();
                Object value = e.getValue();
                if (value == null) {
                    if ((value = map.remove(field)) != null) {
                        changed = true;
                        if (debug > 1) {
                            out.println("## Removed "+field.getName()+" value \""+value+"\"");
                        }
                    }
                } else {
                    Object oldvalue = map.putUntyped(field, value);
                    if (!value.equals(oldvalue)) {
                        changed = true;
                        if (debug > 1) {
                            out.println("## Changed "+field.getName()+" from \""+oldvalue+"\" to \""+value+"\"");
                        }
                    }
                }
            }
            if (changed && (dryrun || ((TrackWriter)reader).write(file, map))) {
                out.println("# Updated \""+file+"\"");
            } else {
                if (debug > 0) out.println("# Processed \""+file+"\"");
            }
        } else {
            out.println("# WARNING: No writer for \""+file+"\"");
        }
    }

    public void close() {
    }

}
