package org.qtunes.telnetd;

import org.qtunes.core.*;
import org.qtunes.auth.*;
import org.qtunes.auth.control.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.net.*;

/**
 * A Simple controller that uses the System.in and System.out to read
 * commands. It accepts "help" to list available commands, and the
 * commands are formatted as you would call a method in JavaScript,
 * eg <code>qtunes.quit()</code>, <code>AppleServer.addRemote("*", "8192")</code>
 * or even <code>db.getFields([1,2,3,4],["Artist","Title","Album"])</code>
 */
public class TelnetController implements Service, Runnable {

    private ServiceContext context;
    private Session session;
    private ControlMethodInvoker invoker;
    private ParamParser parser;
    private boolean cancelled;
    private Thread thread;
    private ServerSocket serversocket;

    public void startService(ServiceContext context) {
        this.context = context;
        cancelled = false;
        int port = Integer.parseInt(context.getProperty("port"));
        try {
            serversocket = new ServerSocket(port);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        thread = new Thread(this, "qTunes-"+context.getServiceName());
        thread.start();
    }

    public void stopService(ServiceContext context) {
        cancelled = true;
        thread.interrupt();
    }

    public ServiceContext getContext() {
        return context;
    }

    public void run() {
        while (!cancelled) {
            try {
                final Socket clientsocket = serversocket.accept();
                Thread thread = new Thread() {
                    public void run() {
                        TelnetConnection con = null;
                        try {
                            con = new TelnetConnection(clientsocket.getInputStream(), clientsocket.getOutputStream(), (InetSocketAddress)clientsocket.getRemoteSocketAddress(), TelnetController.this);
                            if (con.initialize()) {
                                while (con.parseLine());
                            }
                        } catch (IOException e) {
                            context.warn(null, e);
                        } finally {
                            try { clientsocket.getInputStream().close(); } catch (Exception e) {}
                            try { clientsocket.getOutputStream().close(); } catch (Exception e) {}
                            if (con!=null) {
                                try { con.destroy(); } catch (IOException e) { context.warn(null, e); }
                            }
                        }
                    }
                };
                thread.start();
            } catch (IOException e) { 
                if (!cancelled) {
                    context.warn(null, e);
                }
            }
        }
    }

    Session getSession(String username, String password, InetSocketAddress address) {
        Auth auth = context.getService(Auth.class);
        Map<String,Object> props = new HashMap<String,Object>();
        props.put("user", username);
        props.put("password", password);
        props.put("ttl", "0");
        props.put("service", this);
        Session session = auth.login(props);
        return session;
    }

    public Map<String,Object> reportState() {
        return null;
    }

}
