package org.qtunes.web.spi.sun;

import org.qtunes.core.*;
import org.qtunes.web.*;
import java.util.*;
import java.io.*;
import java.net.*;
import java.util.concurrent.*;
import com.sun.net.httpserver.*;

public class SunServer implements WebService {

    private HttpServer server;
    private ServiceContext context;
    private Set<String> reghandlers;
    private String serviceprefix;

    public void setHandler(final String prefix, final WebHandler handler) {
        if (handler!=null) {
            server.createContext(prefix, new HttpHandler() {
                public void handle(HttpExchange exchange) throws IOException {
                    Thread thread = Thread.currentThread();
                    thread.setName("qTunes-"+serviceprefix+prefix+"#"+thread.getId());
                    SunConnection con = new SunConnection(exchange, SunServer.this);
                    String serverheader = context.getProperty("serverheader");
                    if (serverheader != null) {
                        con.setResponseHeader("Server", serverheader);
                    }
                    context.debug(con.getMethod()+" "+con.getRequestURI());
                    try {
                        handler.handle(con);
                    } catch (Exception e) {
                        String m = e.getMessage();
                        if (e instanceof RuntimeException || (e instanceof IOException && !"Broken pipe".equals(m) && !"Connection reset by peer".equals(m) && !"insufficient bytes written to stream".equals(m) && !"stream is closed".equals(m))) {
                            context.warn("Error for \""+con.getRequestURI()+"\"", e);
                        }
                        try {
                            con.sendResponseHeaders(500, -1);
                        } catch (Exception e2) {}
                        if (e instanceof RuntimeException) {
                            throw (RuntimeException)e;
                        } else {
                            throw (IOException)e;
                        }
                    }
                }
            });
            reghandlers.add(prefix);
        } else {
            server.removeContext(prefix);
            reghandlers.remove(prefix);
        }
    }

    public InetSocketAddress getAddress() {
        return server.getAddress();
    }


    public String hasHandler(String path) {
        for (Iterator<String> i = reghandlers.iterator();i.hasNext();) {
            String reg = i.next();
            if (path.startsWith(reg)) {
                return reg;
            }
        }
        return null;
    }

    //---------------------------------------------------------------------------

    public ServiceContext getContext() {
        return context;
    }

    public void startService(ServiceContext context) {
        this.context = context;
        reghandlers = new HashSet<String>();
        int port = Integer.parseInt(context.getProperty("port"));
        try {
            server = HttpServer.create(new InetSocketAddress(port), 0);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        this.serviceprefix = "web:"+port;
        server.setExecutor(Executors.newCachedThreadPool(new ThreadFactory() {
            public Thread newThread(Runnable r) {
                return Executors.defaultThreadFactory().newThread(r);
            }
        }));
        server.start();
    }

    public void stopService(ServiceContext context) {
        server.stop(1);
        reghandlers.clear();
    }

    public Map<String,Object> reportState() {
        Map<String,Object> map = new LinkedHashMap<String,Object>();
        map.put("port", getAddress().getPort());
        return map;
    }

}
